#include <rtt/OperationCaller.hpp>

#include "common.hpp"

bool test_props(RTT::TaskContext &tc)
{
  int nb_it = 6;

  RTT::OperationCaller<double(void)> getDiskUsage;
  RTT::SendHandle     <double(void)> getDiskUsageHandle;

  getDiskUsage = tc.getOperation("getDiskUsage");
  if(!getDiskUsage.ready())
    {
      fprintf(stderr, "%s peer lack a \"getDiskUsage\" operation\n", tc.getName().c_str());
      return false; 
    }

  while(nb_it--)
    {
#define OPT 3

#if (OPT == 1)
      fprintf(stderr, "val = %f\n", getDiskUsage());
#elif (OPT == 2)
      getDiskUsageHandle = getDiskUsage.send();
      double ret;
      SendStatus ss = getDiskUsageHandle.collect(ret);
      if (ss != SendSuccess) 
        {
          fprintf(stderr, "Execution of getDiskUsageHandle failed.");
        }
      fprintf(stderr, "val = %f\n", ret);
#elif (OPT == 3)
      getDiskUsageHandle = getDiskUsage.send();  
      double ret;
      while (getDiskUsageHandle.collectIfDone(ret) == RTT::SendNotReady )
        {
          sleep(1);
        }
      fprintf(stderr, "val = %f\n", ret);
#elif (OPT == 4)
      if(!getDiskUsageHandle.ready())
        {
          //fprintf(stderr, "%d\n", __LINE__);
          getDiskUsageHandle = getDiskUsage.send();
        }
      else
        {
          //fprintf(stderr, "%d\n", __LINE__);
          double ret;
          switch(getDiskUsageHandle.collectIfDone(ret))
            {
            case RTT::SendSuccess:
              fprintf(stderr, "val = %f\n", ret);
            case RTT::SendFailure:
              getDiskUsageHandle = getDiskUsage.send();
              break;
            default:
              break;
            }
        }
#endif

      sleep(1);
    }

  return true;
}

bool test_ports(RTT::TaskContext &tc)
{
  RTT::InputPort<double> port("input");

  // ports
  RTT::base::PortInterface *remote_port = tc.ports()->getPort("output");
  if(remote_port)
    {
      RTT::ConnPolicy policy = RTT::ConnPolicy::data(RTT::ConnPolicy::LOCK_FREE, true, true);
      port.connectTo(remote_port, policy);
      if(!port.connected())
        {
          fprintf(stderr, "can't connect to \"output\" port of peer %s\n", tc.getName().c_str());
          return false;
        }
    }
  else
    {
      fprintf(stderr, "%s peer lack a \"output\" port\n", tc.getName().c_str());
      return false;
    }
 
  return true;
}
